#! /usr/bin/perl 
################################################################################
# Collage Generator Version 1.0
# Written by Suramya Tomar (suramya@suramya.com)
# http://www.suramya.com
# Released Under the GPL Ver 2.
#
# Original code for Yahoo Image search written by Siddharth Uppal (siddhuATgmailDOTcom)
#
#----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or    
# (at your option) any later version.     
#         
# This program is distributed in the hope that it will be useful, but  
# WITHOUT ANY WARRANTY; without even the implied warranty of   
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  
# General Public License for more details.    
#         
# You should have received a copy of the GNU General Public License  
# along with this program; if not, write to the    
# Free Software Foundation, Inc.,      
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   
#########################################################################

#################################################################
# Yahoo Search API Page:										#
# http://developer.yahoo.com/search/image/V1/imageSearch.html	#
#################################################################


#use strict;
#use warnings 'all';
use Imager;
use XML::Simple;
use LWP::UserAgent;

# We read the input from the command line and check if we have all the required parameters
my $numArgs;
my $query;
my $ResultFile;
my $results_quantity;
my $allow_adult;


$numArgs = $#ARGV + 1;

$Instructions = "
\nCollageGenerator.pl is missing file operands.

Usage: CollageGenerator.pl SearchTerm OutputImage.jpg <result quantity> <allow adult results>

Parameters:

SearchTerm       : Seach term using which a collage is created. (Required)
OutputImage.jpg  : Name of the output file where the resultant collage is saved. (Required)

Optional Parameters:

results_quantity : No of images to download for creating the collage. Default 10, Max 50 (Optional)
allow_adult      : The script filters out adult content. Enter a 1 to allow adult content. Default '0' (Optional)

Examples:

CollageGenerator.pl query=roses collage.jpg

will create a image called collage.jpg and use the images returned by an Image search for 'roses' to create a collage.

CollageGenerator.pl girls collage.jpg 30 1

will create an collage using the upto 30 images returned by a search for 'girls' including any adult content

";

if ($numArgs lt 2) # We didn't get all the parameters so print an error message and exit.
{
	print $Instructions;
	exit;
}
else             # We got at least 2 parameters so we need to make sure that we got the required ones.
{
	print "Hi";
	$query = $ARGV[0];
	$ResultFile = $ARGV[1];
	if($numArgs gt 2)
	{
		$results_quantity = $ARGV[2];
	}
	else
	{
		$results_quantity = 10;
	}

	print "results_quantity : $results_quantity \n";

	if($numArgs>3)
	{
		$allow_adult = $ARGV[3];
	}
	else
	{
		$allow_adult = 0;
	}

	print "allow_adult: $allow_adult\n";
}




# APPID: Is the Application ID. Each Application has to have a unique Application ID. 

my $APPID = "NSDul1zV34GBSDULTi0eZfKfs8wM_7q5i.2X4BQJyrQD2NpTNrSB1.zYY11PSQ--";


# Here we are calling the the Yahoo Search API with a random starting no so that we have different 
# images each time we call the Collage generator

my $rand_start = int(rand 10);
my $BaseURL = "http://search.yahooapis.com/ImageSearchService/V1/imageSearch?appid=$APPID&";
my $Parameters = "query=$query&start=$rand_start&results=$results_quantity&adult_ok=$allow_adult";

my $URL = $BaseURL . $Parameters ;

#print "\nURL: $URL\n";

my $ua = LWP::UserAgent->new();
my $response = $ua->get($URL);
# Check for HTTP error codes

if($response->code < 200 || $response->code > 399) 
{
	die 'http status: ' . $response->code . ' ' . $response->message;
}

# Save the response for further processing.

$file = $response->content();

# Convert the XML returned to Perl Array's for processing
my $doc = XMLin($file, ForceArray => 1);

# Get the actual no of results returned
$ResultsReturned = $doc -> {totalResultsReturned};
$counter = 0;

# Create the temp directory

my $test =  rand time(); 
my $directory = "Data/$test";
my $command = "mkdir -p $directory";
`$command`;

# Switch to the temp directory
chdir($directory);

# Once we get the results list. We look through it and download each image

for ($i=0; $i<$ResultsReturned ;$i++) 
{
	$download_url = $doc->{Result}[$i]->{Url}[0];
	print  "Downloading: $download_url . \n";
	download_images($download_url, $counter);
	$counter ++;
}

# Create the Collage
	
create_collage();
print "\nEnd: " . `date`;
exit;




#############################################################################
# Download_Images:                                                          #
#                 Used to download all the images returned by the Yahoo API #
#############################################################################

sub download_images
{
	my $url = shift;
	my $counter = shift;

	my $filename = $counter . ".jpg";

	# Create user agent
	my $u = LWP::UserAgent -> new;

	# Create request
	my $r = HTTP::Request -> new( GET => "$url" );

	# Pass request to UA
	my $ret = $u -> request( $r );

	print "\nError fetching $url" if $ret -> is_error();
	open (OUTFILE, ">$filename");
	print OUTFILE $ret->content;
	close (OUTFILE);
}

#############################################################################################################
# 'create_collage':                                                                                         #
#                   Used to create a collage out of the images downloaded by the 'download_images' function #
#############################################################################################################

sub create_collage
{
	$img=Imager->new();                         # create empty image
	$output = Imager->new(xsize=>640, ysize=>480);

	for ($i=10; $i>0;$i--) 
	{
		opendir(DIR, ".");
		while ($file=readdir(DIR)) 
		{
			if($file =~ /\.jpg/)
			{
				$img->open(file=>$file,type=>'jpeg') or # read image from file
					 goto SKIP_FILE;

				# Choose a random portion of the image to crop

				$top = int(rand($img->getheight()/2));
				$left = int(rand($img->getwidth()/2));
				$width = int(rand($img->getwidth()/2));

				$newimg = $img->crop(left=>$left, top=>$top, width=>$width, height=>$width); 

				# Now we paste this random area into our output image

				$out_top = int(rand(640)/$i) + 10;
				$out_left = int(rand(480)/$i) + 10;

				$output->paste(left=>$out_left, top=>$out_top, img=>$newimg);
			}
			SKIP_FILE: 	  # Need this to avoid the occasional malformed Jpeg image
		}# End of the while Loop
		
		closedir(DIR);
	} # End of For loop

	# Repeating the croping process to get the other half of the image. This reduces the possibility of half empty collages

	for ($i=5; $i>0;$i--) 
	{
		opendir(DIR, ".");
		while ($file=readdir(DIR)) 
		{
			if($file =~ /\.jpg/)
			{
				$img->open(file=>$file,type=>'jpeg') or # read image from file
					goto SKIP_FILE_1;

				# Choose a random portion of the image to crop

				$top = int(rand($img->getheight()/2));
				$left = int(rand($img->getwidth()/2));
				$width = int(rand($img->getwidth()));

				$newimg = $img->crop(left=>$left, top=>$top, width=>$width, height=>$width); 

				# Now we paste this random area into our output image

				$out_top = 640 - int(rand(640));
				$out_left = 480 - int(rand(480));

				$output->paste(left=>$out_left, top=>$out_top, img=>$newimg);
			} #End of If
			SKIP_FILE_1: 	  
		}# End of While Loop
		closedir(DIR);
	}# End of for Loop

	$file_name = "../../$ResultFile";
	$output->write(file=>$file_name, type=>'jpeg');
}
